<?php
/*
 * Comingsoon Action Plugin.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda <kazm@shorindo.com>
 */

class action_plugin_comingsoon extends DokuWiki_Action_Plugin {

	function getInfo() {
        return confToHash(dirname(__FILE__).'/plugin.info.txt');
	}

	function register(&$controller) {
        $controller->register_hook('ACTION_ACT_PREPROCESS',
                                   'AFTER', $this, 'hide_coming');
        $controller->register_hook('IO_WIKIPAGE_WRITE',
                                   'BEFORE', $this, 'remove_coming');
        $controller->register_hook('TPL_ACT_RENDER',
                                   'AFTER', $this, 'show_coming');
	}

    /**
     * Hide comingsoon page.
     */
    function hide_coming(&$event) {
        global $conf;
        $metas = $this->getmeta();
        $delm = $conf['hidepages'] ? '|' : '';
        if (!$metas) return;
        foreach ($metas as $key => $val) {
            $conf['hidepages'] .= $delm.'('.$key.')';
            $delm = '|';
        }
    }

    /**
     * Remove comingsoon meta data.
     */
    function remove_coming(&$event) {
        $text = $event->data[0][1];
        if ($this->getmeta(getID()) && !preg_match('/\{\{comingsoon\}\}/', $text)) {
            $this->removemeta(getID());
        }
    }

    /**
     * Show comingsoon.
     */
    function show_coming(&$event) {
        global $ACT;
        if ($ACT == 'show' && $this->getmeta(getID())) {
            $text = ob_get_contents();
            ob_clean();
            print
                '<div class="comingsoon">'.
                '<img src="'.$this->get_icon().'" alt="COMING SOON"/>'.
                '</div>';
            if (auth_quickaclcheck(getID()) >= AUTH_EDIT)
                print $text;
        }
    }

    /**
     * Get icon image URL at random.
     */
    function get_icon() {
        $files = array();
        $dir = opendir(dirname(__FILE__).'/images/');
        while ($file = readdir($dir)) {
            if (preg_match('/\.(gif|jpe?g|png)$/', $file))
                $files[] = $file;
        }
        $idx = rand(0, count($files)-1);
        return DOKU_URL.'lib/plugins/comingsoon/images/'.$files[$idx];
    }

    /**
     * Remove pageid from meta file if exists.
     */
    function removemeta($id) {
        $fname = metaFN('', '_commingsoon.txt');
        io_lock($fname);
        if (file_exists($fname)) {
            $meta = unserialize(file_get_contents($fname));
            unset($meta[$id]);
        } else {
            $meta = array();
        }
        $fh = @fopen($fname, "wb");
        if (!$fh) {
            io_unlock($fname);
            return false;
        }
        fwrite($fh, serialize($meta));
        fclose($fh);
        io_unlock($fname);
        return true;
    }

    /**
     * Read pageid from meta file.
     */
    function getmeta($id=null) {
        $fname = metaFN('', "_commingsoon.txt");
        if (!@file_exists($fname)) {
            return 0;
        }
        $meta = unserialize(file_get_contents($fname));
        if ($id == null)
            return $meta;
        else
            return $meta[$id];
    }
}
